// 
// 	v 1.1 by Osikani, koganam@nc.rr.com November 6, 2008
//
//=====================================================================================================
//	This is an adjustable version of the Coppock Indicator developed by Edwin Coppock.
//
//	Originally this indicator was designed, and was a correct indicator, for monthly analysis only, 
//	and the indicator had fixed parameters. This one has been modified to plot on weekly and daily charts.
//
//	With the provided override capability, you can use custom parameters to optimize the indicator, 
//	if the original fixed parameters do not seem suitable.
//
//  Feel free to email enhancement suggestions to:
//
//			Osikani, koganam@nc.rr.com
//=====================================================================================================

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Indicator originally developed  by Edwin Coppock.
    /// </summary>
    [Description("Indicator originally developed by Edwin Coppock, to determine when a market returns to a bullish phase.")]
    [Gui.Design.DisplayName("Coppock")]
    public class Coppock : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int rOCPeriod1 = 14; // Default setting for ROCPeriod1
            private int rOCPeriod2 = 11; // Default setting for ROCPeriod2
            private int wMAPeriod = 10; // Default setting for WMAPeriod
        	private DataSeries sum = null;
			private bool	useroverride = false;
			private bool	isInitiated = false;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.MidnightBlue), PlotStyle.Line, "Plot0"));
            Add(new Line(Color.FromKnownColor(KnownColor.Red), 0, "ZeroLine"));
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= true;
			sum = new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			
			// Assuming 52 weeks of 5 trading days, 12 months = 260 days. Subtract 10 days for holidays.
			
			if (!isInitiated)
			{
			double multFactor; 
				
			if (!useroverride) 
				{
				ROCPeriod1 = 14; 
				ROCPeriod2 = 11; 
				WMAPeriod = 10;
				}
			
			if (Bars.Period.Id == PeriodType.Month)	multFactor = 1; 
			else if (Bars.Period.Id == PeriodType.Week) multFactor = 52.0/12.0; //52 weeks = 12 months. Must be explitly 
																				//input as a double value, hence the .0
			else if (Bars.Period.Id == PeriodType.Day) multFactor = 250.0/12.0; //250 days = 12 months
				
			else
				{
					int barsAgo = 0;
					if (CurrentBar > 50) barsAgo = 50;
					DrawOnPricePanel = false;
					DrawText("unsupported", false, "This indicator is supported only on monthly, weekly or daily time frames !", barsAgo, 0.5, Color.Red, new Font("Arial", 20), StringAlignment.Center, Color.Black, Color.Black, 5);
					DrawOnPricePanel = true;
					return;
				}
			
			if (useroverride) multFactor = 1;
			
			ROCPeriod1 = (int) ((double)ROCPeriod1 * multFactor); 
			ROCPeriod2 = (int) ((double)ROCPeriod2 * multFactor);
			WMAPeriod = (int) ((double)WMAPeriod * multFactor);
			
			isInitiated = true;
			}

			sum.Set(ROC(Input, ROCPeriod1)[0] + ROC(Input, ROCPeriod2)[0]);
			Plot0.Set(WMA(sum, WMAPeriod)[0]);
			
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

        [Description("")]
        [Category("Parameters")]
        public int ROCPeriod1
        {
            get { return rOCPeriod1; }
            set { rOCPeriod1 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int ROCPeriod2
        {
            get { return rOCPeriod2; }
            set { rOCPeriod2 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int WMAPeriod
        {
            get { return wMAPeriod; }
            set { wMAPeriod = Math.Max(1, value); }
        }
		
		[Description("Overide automatic calculation of Coppock values. This allows you to use custom values for any of the valid time frames. "
					+ "Note that the original Coppock indicator had fixed values that were invariant.")]
        [Category("Parameters")]
		[Gui.Design.DisplayName("\tUser Override")]
        public bool xUseroverride
        {
            get { return useroverride; }
            set { useroverride = value; }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Coppock[] cacheCoppock = null;

        private static Coppock checkCoppock = new Coppock();

        /// <summary>
        /// Indicator originally developed by Edwin Coppock, to determine when a market returns to a bullish phase.
        /// </summary>
        /// <returns></returns>
        public Coppock Coppock(int rOCPeriod1, int rOCPeriod2, int wMAPeriod, bool xUseroverride)
        {
            return Coppock(Input, rOCPeriod1, rOCPeriod2, wMAPeriod, xUseroverride);
        }

        /// <summary>
        /// Indicator originally developed by Edwin Coppock, to determine when a market returns to a bullish phase.
        /// </summary>
        /// <returns></returns>
        public Coppock Coppock(Data.IDataSeries input, int rOCPeriod1, int rOCPeriod2, int wMAPeriod, bool xUseroverride)
        {
            checkCoppock.ROCPeriod1 = rOCPeriod1;
            rOCPeriod1 = checkCoppock.ROCPeriod1;
            checkCoppock.ROCPeriod2 = rOCPeriod2;
            rOCPeriod2 = checkCoppock.ROCPeriod2;
            checkCoppock.WMAPeriod = wMAPeriod;
            wMAPeriod = checkCoppock.WMAPeriod;
            checkCoppock.xUseroverride = xUseroverride;
            xUseroverride = checkCoppock.xUseroverride;

            if (cacheCoppock != null)
                for (int idx = 0; idx < cacheCoppock.Length; idx++)
                    if (cacheCoppock[idx].ROCPeriod1 == rOCPeriod1 && cacheCoppock[idx].ROCPeriod2 == rOCPeriod2 && cacheCoppock[idx].WMAPeriod == wMAPeriod && cacheCoppock[idx].xUseroverride == xUseroverride && cacheCoppock[idx].EqualsInput(input))
                        return cacheCoppock[idx];

            Coppock indicator = new Coppock();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.ROCPeriod1 = rOCPeriod1;
            indicator.ROCPeriod2 = rOCPeriod2;
            indicator.WMAPeriod = wMAPeriod;
            indicator.xUseroverride = xUseroverride;
            indicator.SetUp();

            Coppock[] tmp = new Coppock[cacheCoppock == null ? 1 : cacheCoppock.Length + 1];
            if (cacheCoppock != null)
                cacheCoppock.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheCoppock = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Indicator originally developed by Edwin Coppock, to determine when a market returns to a bullish phase.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Coppock Coppock(int rOCPeriod1, int rOCPeriod2, int wMAPeriod, bool xUseroverride)
        {
            return _indicator.Coppock(Input, rOCPeriod1, rOCPeriod2, wMAPeriod, xUseroverride);
        }

        /// <summary>
        /// Indicator originally developed by Edwin Coppock, to determine when a market returns to a bullish phase.
        /// </summary>
        /// <returns></returns>
        public Indicator.Coppock Coppock(Data.IDataSeries input, int rOCPeriod1, int rOCPeriod2, int wMAPeriod, bool xUseroverride)
        {
            return _indicator.Coppock(input, rOCPeriod1, rOCPeriod2, wMAPeriod, xUseroverride);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Indicator originally developed by Edwin Coppock, to determine when a market returns to a bullish phase.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Coppock Coppock(int rOCPeriod1, int rOCPeriod2, int wMAPeriod, bool xUseroverride)
        {
            return _indicator.Coppock(Input, rOCPeriod1, rOCPeriod2, wMAPeriod, xUseroverride);
        }

        /// <summary>
        /// Indicator originally developed by Edwin Coppock, to determine when a market returns to a bullish phase.
        /// </summary>
        /// <returns></returns>
        public Indicator.Coppock Coppock(Data.IDataSeries input, int rOCPeriod1, int rOCPeriod2, int wMAPeriod, bool xUseroverride)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Coppock(input, rOCPeriod1, rOCPeriod2, wMAPeriod, xUseroverride);
        }

    }
}
#endregion
